#import
from  matplotlib import pyplot as plt
import numpy as np
from scipy.optimize import fsolve
from scipy import integrate
import math
from mpl_toolkits import mplot3d
import numpy as np
import matplotlib.pyplot as plt


#start conditions *EDIT*    xi   eta phi  p_xi p_eta p_phi
initial_values = np.array([.7, .5, .5, 3., .8, 6.0])


#steps and how long
nsteps = 1000
final= 10
dt = final/nsteps


#physics constants
mu = 3.1
e = -20
hbar = 1.05457266*(10**(-27))
h=hbar*2*math.pi
epsilon = 0.3
realisticmu = 9.1044*(10**(-28))
realistice = 4.80320425*(10**(-10))


#calculate all motion of the particle: evaluate derivates, create time array, use ODE solver, turn array into lists
#evaluate derivatives using equations of motion from partials of hamiltonian
def evaluate(pq,t):
    xi,eta,phi,p_xi,p_eta,p_phi=pq
    xidot  = 4*p_xi*xi/(mu*(eta+xi))
    etadot = 4*p_eta*eta/(mu*(eta+xi))
    phidot = p_phi/(eta*mu*xi)
    p_xidot =  -(e*epsilon)/2-(2*(e**2)/(eta + xi)**2)-((-(p_phi**2/(eta*xi**2))-(4*p_eta**2*eta/(eta+xi)**2)-(4*p_xi**2*xi/(eta+xi)**2)+(4* p_xi**2/(eta+xi)))/(2*mu)) 
    p_etadot = (e*epsilon)/2-(2*(e**2)/(eta + xi)**2)-((-(p_phi**2/(eta**2*xi))-(4*p_eta**2*eta/(eta+xi)**2)-(4*p_xi**2*xi/(eta+xi)**2)+(4*p_eta**2/(eta+xi)))/(2*mu)) 
    p_phidot = 0
    return[xidot,etadot,phidot,p_xidot,p_etadot,p_phidot]

#create time values array
t = np.linspace(0,final,nsteps)

#use ODE solver to calculate all values
pq_t = integrate.odeint(evaluate, initial_values, t)

#turn array into variable lists
swappedpq_t=pq_t.swapaxes(0,1)
xi = swappedpq_t[0]
eta = swappedpq_t[1]
phi = swappedpq_t[2]
p_xi = swappedpq_t[3]
p_eta = swappedpq_t[4]
p_phi = swappedpq_t[5]
#numerically calculate dot variable lists
dt = final/(nsteps-1)
xidot = np.append((xi[1:]-xi[:-1])/dt,[0])
etadot = np.append((eta[1:]-eta[:-1])/dt,[0])
phidot = np.append((phi[1:]-phi[:-1])/dt,[0])
p_xidot = np.append((p_xi[1:]-p_xi[:-1])/dt,[0])
p_etadot = np.append((p_eta[1:]-p_eta[:-1])/dt,[0])
p_phidot = np.append((p_phi[1:]-p_phi[:-1])/dt,[0])
    

#GRAPH ORBIT
xaxis = np.sqrt(xi*eta)
yaxis = np.array((xi - eta)/2)
plt.plot(xaxis, yaxis, label = 'Orbit')
midpointx=.5*(max(xaxis)+min(xaxis))
midpointy=.5*(max(yaxis)+min(yaxis))
rangex=(max(xaxis)-min(xaxis))
rangey=(max(yaxis)-min(yaxis))
plt.xlim(midpointx-rangex,midpointx+rangex)
plt.ylim(midpointy-rangey,midpointy+rangey)
plt.xlabel('√(x\N{SUPERSCRIPT TWO}+y\N{SUPERSCRIPT TWO})')
plt.ylabel('Z')
plt.title('Stark Effect Orbit Graph')
plt.legend()
plt.show()








#MAIN CODE ENDS...







#get cartesian values
cartesianx =(xi*eta)**.5*np.cos(phi)
cartesiany =(xi*eta)**.5*np.sin(phi)
cartesianz =(xi-eta)/2
p_cartesianx = mu*(-np.sin(phi)*(eta*xi)**.5*phidot+(.5*np.cos(phi)*(xi*etadot+eta*xidot)/(eta*xi)**.5))
p_cartesiany = mu*(np.cos(phi)*(eta*xi)**.5*phidot+(.5*np.sin(phi)*(xi*etadot+eta*xidot)/(eta*xi)**.5))
p_cartesianz = mu*((xidot-etadot)/2)


#calculate vector values
r  = np.zeros([nsteps,3])
rhat  = np.zeros([nsteps,3])
momentum = np.zeros([nsteps,3])
angularmomentum = np.zeros([nsteps,3])
rungelenz = np.zeros([nsteps,3])
for i in range (nsteps):
    r[i]=[cartesianx[i],cartesiany[i],cartesianz[i]]
    momentum[i]=[p_cartesianx[i],p_cartesiany[i],p_cartesianz[i]]
    angularmomentum[i]=np.cross(r[i],momentum[i])
    rhat[i]=r[i]/(np.sqrt(r[i].dot(r[i])))
    rungelenz[i]=np.cross(momentum[i],angularmomentum[i])-mu*e**2*rhat[i]


def get_eccentricity(axis1,axis2):
    a=max((max(axis1)-min(axis1))/2,(max(axis2)-min(axis2))/2)
    b=min((max(axis1)-min(axis1))/2,(max(axis2)-min(axis2))/2)
    eccentricity=np.sqrt(1-b**2/a**2)
    return eccentricity


#plot 3d orbit
def threedorbit():
    instancetime = float(input('Enter a time up to {} to plot the r,L,Runge-Lenz vectors at that point: '.format(final)))
    instancestep=int(nsteps*instancetime/final)
    ax = plt.axes(projection ='3d')
    zaxis = (xi-eta)/2
    xaxis = np.sqrt(xi*eta)*np.cos(phi)
    yaxis = np.sqrt(xi*eta)*np.sin(phi)
    c = xaxis + yaxis
    ax.scatter(xaxis, yaxis, zaxis)
    ax.scatter(0, 0, 0)
    if (instancestep>=0 and instancestep<nsteps):
        ax.quiver(0, 0, 0, cartesianx[instancestep], cartesiany[instancestep], cartesianz[instancestep], color='r', normalize=False, label='r', arrow_length_ratio=.05)
        ax.quiver(xaxis[instancestep], yaxis[instancestep], zaxis[instancestep], angularmomentum[instancestep][0], angularmomentum[instancestep][1], angularmomentum[instancestep][2], color='y', normalize=True, label='L', arrow_length_ratio=.05)
        ax.quiver(xaxis[instancestep], yaxis[instancestep], zaxis[instancestep], rungelenz[instancestep][0], rungelenz[instancestep][1], rungelenz[instancestep][2], color='g', normalize=True, label='Runge-Lenz', arrow_length_ratio=.05)
        plt.title('3D Orbit With Vectors at t={}'.format(instancetime))
        plt.legend()
    else:
        plt.title('3D Orbit')
    ax.set_xlabel('X')
    ax.set_ylabel('Y')
    ax.set_zlabel('Z')
    plt.show()
#threedorbit()   


#plot bounding parabolas
def plot_parabolas():
     #solve bounding parabola values
    def equations(vars):
        x, y , a , b = vars
        hamiltonian = (1/(2*mu))*((4/(xi[0] + eta[0]))*(p_xi[0]*xi[0]*p_xi[0]) + (4/(xi[0] + eta[0]))*p_eta[0]*eta[0]*p_eta[0] + p_phi[0]**2/(xi[0]*eta[0])) - (2*e**2)/(xi[0] + eta[0]) + (1/2)*e*epsilon*(xi[0] - eta[0])
        eq1 = -x-y+(-2*xi[0]*(p_xi[0]**2)-((p_phi[0]**2)/(2*xi[0])) + (mu*(e**2)) - ((mu*e*epsilon*(xi[0]**2))/2) + mu*hamiltonian*xi[0] - (mu*(e**2)))/(mu*hamiltonian) 
        eq2 = -(x*y)-(p_phi[0]**2)/(2*mu*hamiltonian)
        eq3 = -a-b+(-(-2*xi[0]*(p_xi[0]**2)-((p_phi[0]**2)/(2*xi[0])) + (mu*(e**2)) + ((mu*e*epsilon*(xi[0]**2))/2) + mu*hamiltonian*xi[0]) - (mu*(e**2)))/(mu*hamiltonian)
        eq4 = -(a*b)-(p_phi[0]**2)/(2*mu*hamiltonian)
        return [eq1, eq2, eq3, eq4]
    #plot actual orbit
    xaxis = np.sqrt(xi*eta)
    yaxis = np.array((xi - eta)/2)
    plt.plot(xaxis, yaxis, label = 'Orbit')
    #turn bounding parabola values into function of xaxis to plot
    maxxivalue, minxivalue, maxetavalue, minetavalue = fsolve(equations, (.5, .5,.5,.5))
    q = np.arange(-10.0, 10.0, 0.001)
    z1=(maxxivalue**2 - q**2)/(2*maxxivalue)
    z2=(minxivalue**2 - q**2)/(2*minxivalue)
    z3=(-maxetavalue**2 + q**2)/(2*maxetavalue)
    z4=(-minetavalue**2 + q**2)/(2*minetavalue)
    plt.plot(q,z1, label='max ξ')
    plt.plot(q,z2, label='min ξ')
    plt.plot(q,z3, label='max η')
    plt.plot(q,z4, label='min η')
    #adjustgraph
    midpointx=.5*(max(xaxis)+min(xaxis))
    midpointy=.5*(max(yaxis)+min(yaxis))
    rangex=(max(xaxis)-min(xaxis))
    rangey=(max(yaxis)-min(yaxis))
    plt.xlim(midpointx-2*rangex,midpointx+2*rangex)
    plt.ylim(midpointy-2*rangey,midpointy+2*rangey)
    plt.xlabel('√(x\N{SUPERSCRIPT TWO}+y\N{SUPERSCRIPT TWO})')
    plt.ylabel('Z')
    plt.title('Stark Effect: Bounding Parabolas')
    plt.legend()
    plt.show() 


#calculate alpha constants from orbit
def get_real_alphas():
    alpha1=(1/(2*mu))*((4/(xi + eta))*(p_xi*xi*p_xi) + (4/(xi + eta))*p_eta*eta*p_eta + p_phi**2/(xi*eta)) - (2*e**2)/(xi + eta) + (1/2)*e*epsilon*(xi - eta)
    alpha3 = p_phi
    alpha2 = .5*(4*eta*p_eta**2+alpha3**2/eta-2*mu*e**2-2*mu*alpha1*eta)
    return [alpha1,alpha2,alpha3]


#calculate alpha values from quantum numbers
def get_quantum_alphas(qn1,qn2,qn3):
    n_xi=qn1
    n_eta=qn2
    n_phi=qn3
    n_principal=n_xi+n_eta+n_phi
    I_xi=n_xi*h
    I_eta=n_eta*h
    I_phi=n_phi*h
    alpha1 = -2*(math.pi)**2*mu*e**4/(I_xi+I_eta+I_phi)**2
    alpha2 = mu*e**2*(I_eta-I_xi)/(I_xi+I_eta+I_phi)
    alpha3 = I_phi/(2*math.pi)
    return [alpha1, alpha2, alpha3]


#calculate quantum number results with epsilon
def get_eccentricity_and_energy_from_quantum_numbers_with_electric_field(qn1,qn2,qn3):
    n_xi=qn1
    n_eta=qn2
    n_phi=qn3
    n_principal=n_xi+n_eta+n_phi
    sigma1=1/n_principal*(math.sqrt(n_xi*(n_xi+n_phi)))
    sigma2=1/n_principal*(math.sqrt(n_eta*(n_eta+n_phi)))
    energy = -mu*e**4/(2*n_principal**2*hbar**2)+3*epsilon*hbar**2/(2*e*mu)*(n_xi-n_eta)*n_principal
    if(n_xi==0 or n_eta==0):
        eccentricity=math.sqrt(sigma1**2+sigma2**2)
        return (eccentricity, energy)
    return energy


